{% extends "sveedocuments/board/base.html" %}
{% load i18n %}

{% block board_menu_summary_active %} class="active"{% endblock %}

{% block board_content %}
    <div class="row">
        <div class="small-12 medium-6 columns">
            <div class="panel radius">
                <h3>{% trans "Pages" %}</h3>
                <p><span class="round label">{{ page_count }}</span> {% trans "saved" %}</p>
                <p><span class="round label">{{ page_hidden_count }}</span> {% trans "hidden" %}</p>
            </div>
        </div>
        <div class="small-12 medium-6 columns">
            <div class="panel radius">
                <h3>{% trans "Inserts" %}</h3>
                <p><span class="round label">{{ insert_count }}</span> {% trans "saved" %}</p>
                <p><span class="round label">{{ insert_hidden_count }}</span> {% trans "hidden" %}</p>
            </div>
        </div>
        <div class="small-12 columns">
            <div class="panel radius">
                <h4>5 {% trans "last pages revised" %}</h4>
                <ul class="square">{% for revised in page_last_revised %}
                    <li>
                        <p><strong><a href="{% url 'sveedocuments:page-edit' revised.slug %}">{{ revised.title }}</a></strong></p>
                        <p><em>{% if revised.comment %}{{ revised.comment }}{% else %}{% trans "Empty comment" %}{% endif %}</em></p>
                        <p>On {{ revised.modified }} by {{ revised.author.get_full_name }}</p>
                    </li>
                {% endfor %}</ul>
            </div>
        </div>
    </div>
{% endblock %}
