{% load i18n %}{% spaceless %}
<div class="clearfix">
    <ul class="button-group radius right">
        {% if perms.sveedocuments.change_page %}
            <li><a class="button tiny secondary" href="{% url 'sveedocuments:page-edit' page_instance.slug %}">{% trans "Edit" %}</a></li>
        {% endif %}
        {% if perms.sveedocuments.delete_page %}
            <li><a class="button tiny secondary" href="{% url 'sveedocuments:page-delete' page_instance.slug %}">{% trans "Delete" %}</a></li>
        {% endif %}
        <li><a class="button tiny secondary" href="{% url 'sveedocuments:page-source' page_instance.slug %}">{% trans "Plain text version" %}</a></li>
        {% if pdf_export_enabled %}
            <li><a class="button tiny secondary" href="{% url 'sveedocuments:page-pdf' page_instance.slug %}">{% trans "PDF version" %}</a></li>
        {% endif %}
        {% if attachments %}
        <li><a class="button tiny secondary" href="#attachments">{% trans "Attachments" %}</a></li>
        {% endif %}
    </ul>
</div>
{% endspaceless %}