{% extends "skeleton.html" %}
{% load i18n mptt_tags autobreadcrumb documents_admin %}

{% block menu_active_link_documents %} class="active"{% endblock %}

{% block head_title %}{% current_title_from_breadcrumbs %}{% endblock %}
{% block head_base_links %}{{ block.super }}
    {% include "sveedocuments/assets_css.html" %}
{% endblock %}
{% block head_base_js %}{{ block.super }}
    {% include "sveedocuments/assets_js.html" %}
{% endblock %}

{% block base_content %}<div class="row row-fluid documents">
    <div class="small-12 columns">
        {% autobreadcrumbs_tag %}
    </div>
    
    <div class="small-12 medium-2 columns context-menu">{% spaceless %}

        <ul class="side-nav">
            <li{% block board_menu_summary_active %}{% endblock %}><a href="{% url 'sveedocuments:board' %}">{% trans "Summary" %}</a></li>
            <li{% block board_menu_pages_active %}{% endblock %}><a href="{% url 'sveedocuments:page-index' %}">{% trans "Pages" %}</a></li>
            <li{% block board_menu_inserts_active %}{% endblock %}><a href="{% url 'sveedocuments:insert-index' %}">{% trans "Inserts" %}</a></li>
        </ul>

    {% endspaceless %}</div>
    
    <div class="small-12 medium-10 columns">
        {% block board_content %}{% endblock %}
    </div>
</div>{% endblock %}
